#!/bin/bash

set -e

make_dcstore_xml()
{
	module_type=$1
	rootdir=$2
	git_action=$3

	modules=$(cd "$rootdir" && ls -d -- */ | sed -e "s/\///g")

	for module in $modules; do
		# For each module in root dir

		if [ -d "$rootdir/$module/.git" ] && [ ! -f "$rootdir/$module/.notmine" ]; then
			# There is a git repository here

			dcstore_xml="$rootdir/$module/dcstore.xml"
			define_php="$rootdir/$module/_define.php"

			if [ ! -f "$define_php" ]; then
				echo "Warning, _define.php is missing in $rootdir/$module !"
			else
				status="updated"
				if [ ! -f "$dcstore_xml" ]; then
					status="created"
				fi

				# Extract info from _define.php
				define=$(./mk-dcstore.php "$define_php")

				IFS='£'
				read -a data <<< "$define"

				# Write dcstore.xml
				buffer="$DCSTORE_XML_TEMPLATE"

				details="$DEFAULT_DETAILS"
				if [ "${data[5]}" != "-" ]; then
					details="${data[5]}"
				fi
				buffer=${buffer//\[%details%\]/"$details"}

				support="$DEFAULT_SUPPORT"
				if [ "${data[6]}" != "-" ]; then
					support="${data[6]}"
				fi
				buffer=${buffer//\[%support%\]/"$support"}

				buffer=${buffer//\[%type%\]/"$module_type"}
				buffer=${buffer//\[%id%\]/"$module"}

				buffer=${buffer//\[%name%\]/"${data[0]}"}
				buffer=${buffer//\[%version%\]/"${data[3]}"}
				buffer=${buffer//\[%author%\]/"${data[2]}"}
				buffer=${buffer//\[%description%\]/"${data[1]}"}
				buffer=${buffer//\[%dcmin%\]/"${data[4]}"}

				echo "$buffer" > "$dcstore_xml"

				test=$(cd "$rootdir/$module" && git diff --name-only -- dcstore.xml)
				if [ "$test" != "" ]; then
					echo "dcstore.xml $status for $module_type $module."

					if [ "$git_action" != "" ]; then
						# Add, commit and push if required
						echo $(cd "$rootdir/$module" && git add dcstore.xml)
						if [ "$git_action" != "add" ]; then
							echo $(cd "$rootdir/$module" && git commit -m "Update dcstore.xml")
							if [ "$git_action" != "commit" ]; then
								echo $(cd "$rootdir/$module" && git push)
							fi
						fi
					fi
				else
					echo "nothing to do for $module_type $module."
				fi
			fi
		fi

	done
}

DEFAULT_DETAILS="https://open-time.net/?q=[%id%]"
DEFAULT_SUPPORT="https://github.com/franck-paul/[%id%]"

DCSTORE_XML_TEMPLATE=$(cat <<-END
	<modules xmlns:da="http://dotaddict.org/da/">
	  <module id="[%id%]">
	    <name>[%name%]</name>
	    <version>[%version%]</version>
	    <author>[%author%]</author>
	    <desc>[%description%]</desc>
	    <file>https://github.com/franck-paul/[%id%]/releases/download/[%version%]/[%type%]-[%id%]-[%version%].zip</file>
	    <da:dcmin>[%dcmin%]</da:dcmin>
	    <da:details>[%details%]</da:details>
	    <da:support>[%support%]</da:support>
	  </module>
	</modules>
END
)

case $1 in
	-t | -T)
		ROOT_DIR="$HOME/Documents/EGO/Geekeries/Internet/htdocs/git-themes"
		MODULE_TYPE="theme"
		;;
	-p | -P)
		ROOT_DIR="$HOME/Documents/EGO/Geekeries/Internet/htdocs/git-plugins"
		MODULE_TYPE="plugin"
		;;
	-h | -H)
		echo "run $0 -<type> [-<action>]"
		echo ""
		echo "-<type>: type of modules"
		echo "p: for plugins"
		echo "t: for themes"
		echo ""
		echo "-<action>: git action"
		echo "n: none"
		echo "a: for add only"
		echo "c: for add and commit"
		echo "other (or no argument): add, commit and push"
		echo ""
		echo "exemple:"
		echo "$0 -p -a"
		echo "will run in the plugins directory and only git add the created/updated dcstore.xml"
		exit 0
		;;
	*)
		echo "Parameters missing (run $0 -h for help)"
		exit 1
		;;
esac

case $2 in
	-n | -N)
		GIT=""
		;;
	-a | -A)
		GIT="add"
		;;
	-c | -C)
		GIT="commit"
		;;
	*)
		GIT="all"
		;;
esac

echo "Directory : $ROOT_DIR ($MODULE_TYPE)"

make_dcstore_xml "$MODULE_TYPE" "$ROOT_DIR" "$GIT"
